/*                                                         */
/*       SUBROUTINE FOR DCF3.C  <udcf.c>   ver 1.1         */
/*                                                         */
/*          Made on     92.9.22            by C. Otani     */
/*          modified on 92.10.3                            */
/*         modified for  tempd = b[k2].tim - a[k1].tim     */
/*                 ISAS 981207      Jun  Kataoka           */
int udcfsub(RAW a[], RAW b[], int na, int nb, long *nab){
  int          k, k1, k2;
  long         kl, kl1, kl2;
  double       a_, b_, s_a, s_b, e_a, e_b, norm;
  double       tempt, tempd;
  char         fname[50], fname2[50], command[100];
  FILE         *fp;


/*   initialization   */
  printf("\n\nUDCF Calculation started.\n");
  printf("  na  = %d\n",na);
  printf("  nb  = %d\n",nb);


/*   Calculate Averages, Sigmas and Measurement Errors   */
/*     f_ = "average of f"                               */
/*     s_a = "square of the standard deviation of f"     */
/*     e_a = "square of the measurement error of f"      */
  for ( k=0, a_=0.0, e_a=0.0, s_a=0.0; k<na; k++){
    a_  += (double)a[k].dat;
    s_a += (double)( a[k].dat * a[k].dat );
    e_a += (double)a[k].err;
  }
  for ( k=0, b_=0.0, e_b=0.0, s_b=0.0; k<nb; k++){
    b_  += (double)b[k].dat;
    s_b += (double)(b[k].dat * b[k].dat);
    e_b += (double)b[k].err;
  }
  a_ /= (double)na; e_a /= (double)na; e_a *= e_a;
  b_ /= (double)nb; e_b /= (double)nb; e_b *= e_b;
  s_a = ( s_a - (double)na*a_*a_ ) / (double)(na-1);
  s_b = ( s_b - (double)nb*b_*b_ ) / (double)(nb-1);
  printf("  sigma_a2 = %f\n",s_a);
  printf("  sigma_b2 = %f\n",s_b);
  printf("  error_a2 = %f\n",e_a);
  printf("  error_b2 = %f\n",e_b);


/*   Checking (s_a-e_a)(s_b-e_b)>0 or not.   */
  if ( ( norm = (s_a-e_a)*(s_b-e_b) ) > 0 ){
    norm = sqrt(norm);
    printf("  root[(s_a-e_a)(s_b-e_b)] = %f .\n", norm);
  }
  else{
    printf("  (s_a-e_a)(s_b-e_b) = %e .\n", norm);
    printf("\n  Can't calculate UDCF !!\n");
    return(-1);
  }



/*   to output udcf                        */
/*     tempt = "time lag"                  */
/*     templ = "correlation coefficient"   */
  printf("Output File >");gets(fname);
  printf("\nStart Making a UDCF.\n");
  sprintf(fname2,"%s.w",fname);
  if ( ( fp = fopen(fname2,"w") ) == NULL ){
    printf("\nFile Open Error !!\n");
    return(-1);
  }
  for ( k1=0, kl=0L; k1<na; k1++){
    for ( k2=0; k2<nb; k2++){
      if ( ( tempt = b[k2].tim - a[k1].tim ) != 0.0){
	tempd = ((double)a[k1].dat - a_)*((double)b[k2].dat - b_)/norm;
	fprintf(fp, "%lf\t",tempt);
	fprintf(fp, "%lf\n",tempd);
      }
    }
  }
  fclose(fp);


/*   sorting   */
  sprintf(command, "sort -n %s > %s",fname2,fname);
  system(command);
  sprintf(command,"rm -f %s",fname2);
  system(command);

  return(0);

}

